@@ -2,9 +2,9 @@
 #
 #      cons.py
 #
-#      Copyright 2009-2011
+#      Copyright 2009-2012
 #      Giuseppe Penone <giuspen@gmail.com>,
-#      Chris Camacho (chris_c) <chris_camacho@yahoo.com>.
+#      Chris Camacho (chris_c) <codifies@gmail.com>.
 #
 #      plus many thanks to  http://tronche.com/gui/x/xlib/
 #                      and  http://tripie.sweb.cz/utils/wmctrl/
@@ -28,19 +28,17 @@ import gtk, os
 
 
 APP_NAME = "x-tile"
-VERSION = "2.2.1"
-if os.path.isdir('glade'):
-    GLADE_PATH = "glade/"
-    ICON_PLACE = "linux/x-tile.svg"
+VERSION = "2.5"
+if os.path.isdir("glade"):
+    GLADE_PATH = os.path.join(os.getcwd(), "glade/")
     LOCALE_PATH = "locale/"
 else:
     GLADE_PATH = "/usr/share/x-tile/glade/"
-    ICON_PLACE = "/usr/share/pixmaps/x-tile.svg"
     LOCALE_PATH = "/usr/share/locale/"
 
-AVAILABLE_LANGS = ['default', 'cs', 'de', 'en', 'es_AR', 'fr', 'it', 'ru', 'zh_CN', 'zh_TW']
+AVAILABLE_LANGS = ['default', 'cs', 'de', 'en', 'es_AR', 'fr', 'it', 'pl', 'ru', 'zh_CN', 'zh_TW']
 
-CMD_LINE_ACTIONS = ["z", "i", "v", "h", "u", "d", "l", "r", "1", "2", "q", "g", "m", "M", "c"]
+CMD_LINE_ACTIONS = ["z", "i", "y", "v", "h", "u", "d", "l", "r", "1", "2", "q", "g", "m", "M", "c"]
 
 OVERRIDE_1 = 0
 OVERRIDE_2 = 0
@@ -57,6 +55,7 @@ GRID_COLS = 2
 
 WINNAMES_BLACKLIST = ["x-nautilus-desktop", "pcmanfm"]
 PROCESSES_BLACKLIST = ["lxpanel"]
+#PROCESSES_PARENT_BLACKLIST = ["gnome-session"]
 
 STR_TRUE = "True"
 STR_FALSE = "False"
@@ -80,28 +79,29 @@ DRAW_RGBS = (
 ICONS_SIZE = {1: gtk.ICON_SIZE_MENU, 2: gtk.ICON_SIZE_SMALL_TOOLBAR, 3: gtk.ICON_SIZE_LARGE_TOOLBAR,
               4: gtk.ICON_SIZE_DND, 5: gtk.ICON_SIZE_DIALOG}
 
-ICONS_FILENAMES = [(GLADE_PATH + 'tile-vertically.svg', 'Tile Vertically'),
-                   (GLADE_PATH + 'tile-horizontally.svg', 'Tile Horizontally'),
-                   (GLADE_PATH + 'tile-quad.svg', 'Tile Quad'),
-                   (GLADE_PATH + 'tile-grid.svg', 'Tile Grid'),
-                   (GLADE_PATH + 'tile-custom-1-set.svg', 'Custom Set 1'),
-                   (GLADE_PATH + 'tile-custom-1-exe.svg', 'Custom Exe 1'),
-                   (GLADE_PATH + 'tile-custom-2-set.svg', 'Custom Set 2'),
-                   (GLADE_PATH + 'tile-custom-2-exe.svg', 'Custom Exe 2'),
-                   (GLADE_PATH + 'tile-triangle-up.svg', 'Tile Triangle Up'),
-                   (GLADE_PATH + 'tile-triangle-down.svg', 'Tile Triangle Down'),
-                   (GLADE_PATH + 'tile-triangle-left.svg', 'Tile Triangle Left'),
-                   (GLADE_PATH + 'tile-triangle-right.svg', 'Tile Triangle Right'),
-                   (GLADE_PATH + 'reload-windows-list.svg', 'Reload Windows List'),
-                   (GLADE_PATH + 'maximize-checked-windows.svg', 'Maximize Windows'),
-                   (GLADE_PATH + 'unmaximize-checked-windows.svg', 'Unmaximize Windows'),
-                   (GLADE_PATH + 'close-checked-windows.svg', 'Close Windows'),
-                   (GLADE_PATH + 'invert-tiling.svg', 'Invert Tiling'),
-                   (GLADE_PATH + 'toolbar.png', 'Toolbar'),
-                   (GLADE_PATH + 'quit-app.svg', 'Quit App'),
-                   (GLADE_PATH + 'help-contents.svg', 'Help Contents'),
-                   (GLADE_PATH + 'checkbox_checked.svg', 'Select All'),
-                   (GLADE_PATH + 'checkbox_unchecked.svg', 'Deselect All')]
+ICONS_FILENAMES = ['tile-vertically.svg',
+                   'tile-horizontally.svg',
+                   'tile-quad.svg',
+                   'tile-grid.svg',
+                   'tile-custom-1-set.svg',
+                   'tile-custom-1-exe.svg',
+                   'tile-custom-2-set.svg',
+                   'tile-custom-2-exe.svg',
+                   'tile-triangle-up.svg',
+                   'tile-triangle-down.svg',
+                   'tile-triangle-left.svg',
+                   'tile-triangle-right.svg',
+                   'reload-windows-list.svg',
+                   'maximize-checked-windows.svg',
+                   'unmaximize-checked-windows.svg',
+                   'close-checked-windows.svg',
+                   'invert-tiling.svg',
+                   'cycle-tiling.svg',
+                   'toolbar.png',
+                   'quit-app.svg',
+                   'help-contents.svg',
+                   'checkbox_checked.svg',
+                   'checkbox_unchecked.svg']
 
 GCONF_DIR = "/apps/x-tile"
 GCONF_LANG = "/apps/x-tile/language"
@@ -133,6 +133,7 @@ GCONF_OVERRIDE_2 = "/apps/x-tile/%s/override_2"
 GCONF_GRID_ROWS = "/apps/x-tile/%s/grid_rows"
 GCONF_GRID_COLS = "/apps/x-tile/%s/grid_cols"
 GCONF_SYSTRAY_ENABLE = "/apps/x-tile/%s/systray_enable"
+GCONF_USE_APPIND = "/apps/x-tile/%s/use_appind"
 GCONF_SYSTRAY_START = "/apps/x-tile/%s/systray_start"
 
 UI_INFO = """
@@ -165,6 +166,7 @@ UI_INFO = """
         <menu action='TileMenu'>
             <menuitem action='UnTile'/>
             <menuitem action='InverTile'/>
+            <menuitem action='CycleTile'/>
             <separator/>
             <menuitem action='Vertically'/>
             <menuitem action='Horizontally'/>
@@ -207,6 +209,7 @@ UI_INFO = """
         <separator/>
         <toolitem action='UnTile'/>
         <toolitem action='InverTile'/>
+        <toolitem action='CycleTile'/>
         <separator/>
         <toolitem action='Vertically'/>
         <toolitem action='Horizontally'/>
@@ -242,6 +245,7 @@ UI_INFO = """
         <menuitem action='All_H'/>
         <menuitem action='All_V'/>
         <separator/>
+        <menuitem action='All_Cycle'/>
         <menuitem action='All_Invert'/>
         <menuitem action='All_Undo'/>
         <separator/>
@@ -270,54 +274,56 @@ def get_entries(inst):
     # name, stock id, label, accelerator, tooltip, callback
     ( "Filter", "gtk-properties", _("_Filter"), "<control>F", _("Filter Rows"), inst.dialog_filter),
     ( "DefaultSel", "gtk-properties", _("Selected by _Default"), "<control>D", _("Rows to be Selected by Default"), inst.dialog_selected_by_default),
-    ( "Reload", "Reload Windows List", _("_Reload"), "F5", _("Reload the Windows List"), inst.reload_windows_list),
-    ( "QuitApp", "Quit App", _("_Quit"), "<control>Q", _("Quit the Application"), inst.quit_application),
-    ( "ExitApp", "Quit App", _("_Exit X Tile"), "<control><shift>Q", _("Exit from X Tile"), inst.quit_application_totally),
-    ( "ShowHideMainWin", "Tile Quad", _("Show/Hide _X Tile"), None, _("Toggle Show/Hide X Tile"), inst.toggle_show_hide_main_window),
+    ( "Reload", "reload-windows-list", _("_Reload"), "F5", _("Reload the Windows List"), inst.reload_windows_list),
+    ( "QuitApp", "quit-app", _("_Quit"), "<control>Q", _("Quit the Application"), inst.quit_application),
+    ( "ExitApp", "quit-app", _("_Exit X Tile"), "<control><shift>Q", _("Exit from X Tile"), inst.quit_application_totally),
+    ( "ShowHideMainWin", "tile-quad", _("Show/Hide _X Tile"), None, _("Toggle Show/Hide X Tile"), inst.toggle_show_hide_main_window),
     ( "Preferences", "gtk-preferences", _("_Preferences"), "<control><alt>P", _("Open the Preferences Window"), inst.dialog_preferences),
-    ( "SelectAll", "Select All", _("Select _All"), "<control>A", _("Select All the Windows in the List"), inst.flag_all_rows),
-    ( "DeselectAll", "Deselect All", _("Deselect A_ll"), "<control><shift>A", _("Deselect All the Windows in the List"), inst.unflag_all_rows),
-    ( "Vertically", "Tile Vertically", _("Tile _Vertically"), "<control>V", _("Tile Vertically The Checked Windows"), inst.tile_vertically),
-    ( "Horizontally", "Tile Horizontally", _("Tile _Horizontally"), "<control>H", _("Tile Horizontally The Checked Windows"), inst.tile_horizontally),
-    ( "TriangleUp", "Tile Triangle Up", _("_Triangle Up"), "<control>Up", _("Tile Triangle Up The Checked Windows"), inst.tile_triangle_up),
-    ( "TriangleDown", "Tile Triangle Down", _("Triangle _Down"), "<control>Down", _("Tile Triangle Down The Checked Windows"), inst.tile_triangle_down),
-    ( "TriangleLeft", "Tile Triangle Left", _("Triangle _Left"), "<control>Left", _("Tile Triangle Left The Checked Windows"), inst.tile_triangle_left),
-    ( "TriangleRight", "Tile Triangle Right", _("Triangle _Right"), "<control>Right", _("Tile Triangle Right The Checked Windows"), inst.tile_triangle_right),
-    ( "Quad", "Tile Quad", _("Tile _Quad"), "<control>4", _("Tile into 4 quadrants The Checked Windows"), inst.tile_quad),
-    ( "Grid", "Tile Grid", _("Tile _Grid"), "<control>G", _("Tile into an Arbitrary Grid The Checked Windows"), inst.dialog_grid),
-    ( "Custom1Set", "Custom Set 1", _("Custom Tile 1 _Set"), "<alt>1", _("Edit Custom Tile 1 Settings"), inst.tile_custom_1_set),
-    ( "Custom1Exe", "Custom Exe 1", _("Custom Tile _1 Run"), "<control>1", _("Execute Custom Tile 1"), inst.tile_custom_1_run),
-    ( "Custom2Set", "Custom Set 2", _("Custom Tile 2 S_et"), "<alt>2", _("Edit Custom Tile 2 Settings"), inst.tile_custom_2_set),
-    ( "Custom2Exe", "Custom Exe 2", _("Custom Tile _2 Run"), "<control>2", _("Execute Custom Tile 2"), inst.tile_custom_2_run),
+    ( "SelectAll", "checkbox_checked", _("Select _All"), "<control>A", _("Select All the Windows in the List"), inst.flag_all_rows),
+    ( "DeselectAll", "checkbox_unchecked", _("Deselect A_ll"), "<control><shift>A", _("Deselect All the Windows in the List"), inst.unflag_all_rows),
+    ( "Vertically", "tile-vertically", _("Tile _Vertically"), "<control>V", _("Tile Vertically The Checked Windows"), inst.tile_vertically),
+    ( "Horizontally", "tile-horizontally", _("Tile _Horizontally"), "<control>H", _("Tile Horizontally The Checked Windows"), inst.tile_horizontally),
+    ( "TriangleUp", "tile-triangle-up", _("_Triangle Up"), "<control>Up", _("Tile Triangle Up The Checked Windows"), inst.tile_triangle_up),
+    ( "TriangleDown", "tile-triangle-down", _("Triangle _Down"), "<control>Down", _("Tile Triangle Down The Checked Windows"), inst.tile_triangle_down),
+    ( "TriangleLeft", "tile-triangle-left", _("Triangle _Left"), "<control>Left", _("Tile Triangle Left The Checked Windows"), inst.tile_triangle_left),
+    ( "TriangleRight", "tile-triangle-right", _("Triangle _Right"), "<control>Right", _("Tile Triangle Right The Checked Windows"), inst.tile_triangle_right),
+    ( "Quad", "tile-quad", _("Tile _Quad"), "<control>4", _("Tile into 4 quadrants The Checked Windows"), inst.tile_quad),
+    ( "Grid", "tile-grid", _("Tile _Grid"), "<control>G", _("Tile into an Arbitrary Grid The Checked Windows"), inst.dialog_grid),
+    ( "Custom1Set", "tile-custom-1-set", _("Custom Tile 1 _Set"), "<alt>1", _("Edit Custom Tile 1 Settings"), inst.tile_custom_1_set),
+    ( "Custom1Exe", "tile-custom-1-exe", _("Custom Tile _1 Run"), "<control>1", _("Execute Custom Tile 1"), inst.tile_custom_1_run),
+    ( "Custom2Set", "tile-custom-2-set", _("Custom Tile 2 S_et"), "<alt>2", _("Edit Custom Tile 2 Settings"), inst.tile_custom_2_set),
+    ( "Custom2Exe", "tile-custom-2-exe", _("Custom Tile _2 Run"), "<control>2", _("Execute Custom Tile 2"), inst.tile_custom_2_run),
     ( "UnTile", "gtk-undo", _("U_ndo Tiling"), "<control>Z", _("Undo the Latest Tiling Operation"), inst.undo_tiling),
-    ( "InverTile", "Invert Tiling", _("_Invert Tiling Order"), "<control>I", _("Invert the Order of the Latest Tiling Operation"), inst.invert_tiling),
-    ( "Maximize", "Maximize Windows", _("_Maximize Windows"), "<control>M", _("Maximize The Checked Windows"), inst.maximize_checked_windows),
-    ( "Unmaximize", "Unmaximize Windows", _("_Unmaximize Windows"), "<control>U", _("Unmaximize The Checked Windows"), inst.unmaximize_checked_windows),
-    ( "Close", "Close Windows", _("_Close Windows"), "<control>C", _("Close The Checked Windows"), inst.close_checked_windows),
+    ( "InverTile", "invert-tiling", _("_Invert Tiling Order"), "<control>I", _("Invert the Order of the Latest Tiling Operation"), inst.invert_tiling),
+    ( "CycleTile", "cycle-tiling", _("C_ycle Tiling Order"), "<control>Y", _("Cycle the Order of the Latest Tiling Operation"), inst.cycle_tiling),
+    ( "Maximize", "maximize-checked-windows", _("_Maximize Windows"), "<control>M", _("Maximize The Checked Windows"), inst.maximize_checked_windows),
+    ( "Unmaximize", "unmaximize-checked-windows", _("_Unmaximize Windows"), "<control>U", _("Unmaximize The Checked Windows"), inst.unmaximize_checked_windows),
+    ( "Close", "close-checked-windows", _("_Close Windows"), "<control>C", _("Close The Checked Windows"), inst.close_checked_windows),
     ( "RowUp", "gtk-go-up", _("Move _Up"), "<alt>Up", _("Move the Selected Row Up"), inst.on_button_row_up_clicked),
     ( "RowDown", "gtk-go-down", _("Move _Down"), "<alt>Down", _("Move the Selected Row Down"), inst.on_button_row_down_clicked),
     ( "RowDelete", "gtk-remove", _("_Remove"), "Delete", _("Remove the Selected Row"), inst.on_button_row_delete_clicked),
-    ( "ShowHideToolbar", "Toolbar", _("Show/Hide _Toolbar"), None, _("Toggle Show/Hide Toolbar"), inst.show_hide_toolbar),
+    ( "ShowHideToolbar", "toolbar", _("Show/Hide _Toolbar"), None, _("Toggle Show/Hide Toolbar"), inst.show_hide_toolbar),
     ( "IncreaseToolbarIconsSize", "gtk-zoom-in", _("_Increase Toolbar Icons Size"), None, _("Increase the Size of the Toolbar Icons"), inst.toolbar_icons_size_increase),
     ( "DecreaseToolbarIconsSize", "gtk-zoom-out", _("_Decrease Toolbar Icons Size"), None, _("Decrease the Size of the Toolbar Icons"), inst.toolbar_icons_size_decrease),
-    ( "Help", "Help Contents", _("_Help"), None, _("X Tile Project Home Page"), inst.on_help_menu_item_activated),
+    ( "Help", "help-contents", _("_Help"), None, _("X Tile Project Home Page"), inst.on_help_menu_item_activated),
     ( "About", "gtk-about", _("_About"), None, _("About X Tile"), inst.dialog_about),
     ( "All_About", "gtk-about", _("_About"), None, _("About X Tile"), inst.dialog_about_all),
-    ( "All_C", "Close Windows", _("_Close All"), None, _("Close All Windows"), inst.close_all),
-    ( "All_U", "Unmaximize Windows", _("_Unmaximize All"), None, _("Unmaximize All Windows"), inst.unmaximize_all),
-    ( "All_M", "Maximize Windows", _("_Maximize All"), None, _("Maximize All Windows"), inst.maximize_all),
-    ( "All_4", "Tile Quad", _("Tile All _Quad"), None, _("Tile All Windows Quad"), inst.tile_all_quad),
-    ( "All_TD", "Tile Triangle Down", _("Tile All Triangle _Down"), None, _("Tile All Windows Triangle Down"), inst.tile_all_triangle_down),
-    ( "All_TU", "Tile Triangle Up", _("Tile All Triangle _Up"), None, _("Tile All Windows Triangle Up"), inst.tile_all_triangle_up),
-    ( "All_TR", "Tile Triangle Right", _("Tile All Triangle _Right"), None, _("Tile All Windows Triangle Right"), inst.tile_all_triangle_right),
-    ( "All_TL", "Tile Triangle Left", _("Tile All Triangle _Left"), None, _("Tile All Windows Triangle Left"), inst.tile_all_triangle_left),
-    ( "All_C2", "Custom Exe 2", _("Tile All Custom _2"), None, _("Tile All Windows Custom 2"), inst.tile_all_custom_2),
-    ( "All_C1", "Custom Exe 1", _("Tile All Custom _1"), None, _("Tile All Windows Custom 1"), inst.tile_all_custom_1),
-    ( "All_G", "Tile Grid", _("Tile All _Grid"), None, _("Tile All Windows Grid"), inst.tile_all_grid),
-    ( "All_H", "Tile Horizontally", _("Tile All _Horizontally"), None, _("Tile All Windows Horizontally"), inst.tile_all_horizontally),
-    ( "All_V", "Tile Vertically", _("Tile All _Vertically"), None, _("Tile All Windows Vertically"), inst.tile_all_vertically),
-    ( "All_Invert", "Invert Tiling", _("_Invert Tiling Order"), None, _("Invert the Order of the Latest Tiling Operation"), inst.invert_tiling_all),
+    ( "All_C", "close-checked-windows", _("_Close All"), None, _("Close All Windows"), inst.close_all),
+    ( "All_U", "unmaximize-checked-windows", _("_Unmaximize All"), None, _("Unmaximize All Windows"), inst.unmaximize_all),
+    ( "All_M", "maximize-checked-windows", _("_Maximize All"), None, _("Maximize All Windows"), inst.maximize_all),
+    ( "All_4", "tile-quad", _("Tile All _Quad"), None, _("Tile All Windows Quad"), inst.tile_all_quad),
+    ( "All_TD", "tile-triangle-down", _("Tile All Triangle _Down"), None, _("Tile All Windows Triangle Down"), inst.tile_all_triangle_down),
+    ( "All_TU", "tile-triangle-up", _("Tile All Triangle _Up"), None, _("Tile All Windows Triangle Up"), inst.tile_all_triangle_up),
+    ( "All_TR", "tile-triangle-right", _("Tile All Triangle _Right"), None, _("Tile All Windows Triangle Right"), inst.tile_all_triangle_right),
+    ( "All_TL", "tile-triangle-left", _("Tile All Triangle _Left"), None, _("Tile All Windows Triangle Left"), inst.tile_all_triangle_left),
+    ( "All_C2", "tile-custom-2-exe", _("Tile All Custom _2"), None, _("Tile All Windows Custom 2"), inst.tile_all_custom_2),
+    ( "All_C1", "tile-custom-1-exe", _("Tile All Custom _1"), None, _("Tile All Windows Custom 1"), inst.tile_all_custom_1),
+    ( "All_G", "tile-grid", _("Tile All _Grid"), None, _("Tile All Windows Grid"), inst.tile_all_grid),
+    ( "All_H", "tile-horizontally", _("Tile All _Horizontally"), None, _("Tile All Windows Horizontally"), inst.tile_all_horizontally),
+    ( "All_V", "tile-vertically", _("Tile All _Vertically"), None, _("Tile All Windows Vertically"), inst.tile_all_vertically),
+    ( "All_Invert", "invert-tiling", _("_Invert Tiling Order"), None, _("Invert the Order of the Latest Tiling Operation"), inst.invert_tiling_all),
     ( "All_Undo", "gtk-undo", _("U_ndo Tiling"), None, _("Undo the Latest Tiling Operation"), inst.undo_tiling_all),
+    ( "All_Cycle", "cycle-tiling", _("C_ycle Tiling Order"), None, _("Cycle the Order of the Latest Tiling Operation"), inst.cycle_tiling_all),
     ]
 
 HELP_TEXT = """
@@ -326,7 +332,7 @@ NAME
    X Tile - Tile the Windows upon your Desktop
 
 SYNOPSIS
-   x-tile OPTION [grid rows] [grid columns]
+   x-tile OPTION
 
 DESCRIPTION
    This manual page briefly documents the command line arguments
@@ -351,7 +357,10 @@ OPTIONS
 
    q => quad tile all opened windows
 
-   g => tile all opened windows in a grid having number of rows and columns the given arguments
+   g = g 0 = g 0 0   => tile all opened windows in a grid with automatic rows and columns
+   g rows = g rows 0 => tile all opened windows in a grid with given rows and automatic columns
+   g 0 cols          => tile all opened windows in a grid with automatic rows and given columns
+   g rows cols       => tile all opened windows in a grid with given rows and columns
 
    1 => custom tile 1 all opened windows
 
@@ -359,6 +368,8 @@ OPTIONS
 
    i => invert the order of the latest tiling operation
 
+   y => cycle the order of the latest tiling operation
+
    m => maximize all opened windows
 
    M => unmaximize all opened windows
