@@ -2,9 +2,9 @@
 #
 #      core.py
 #
-#      Copyright 2009-2011
+#      Copyright 2009-2012
 #      Giuseppe Penone <giuspen@gmail.com>,
-#      Chris Camacho (chris_c) <chris_camacho@yahoo.com>.
+#      Chris Camacho (chris_c) <codifies@gmail.com>.
 #
 #      plus many thanks to  http://tronche.com/gui/x/xlib/
 #                      and  http://tripie.sweb.cz/utils/wmctrl/
@@ -30,6 +30,8 @@ try:
     import appindicator
     HAS_APPINDICATOR = True
 except: HAS_APPINDICATOR = False
+XDG_CURRENT_DESKTOP = 'XDG_CURRENT_DESKTOP'
+HAS_SYSTRAY = not (XDG_CURRENT_DESKTOP in os.environ and os.environ[XDG_CURRENT_DESKTOP] == "Unity")
 import cons, support, tilings
 
 
@@ -63,7 +65,6 @@ class InfoModel:
             curr_workspace_num = -2
             print "DEBUG warning _NET_CURRENT_DESKTOP improperly set"
         else: curr_workspace_num = glob.ret_pointer[0] # the number of the current workspace
-        desktop_width, desktop_height = support.get_desktop_width_n_height()
         screen = gtk.gdk.screen_get_default()
         for client in clients:
             if support.is_window_sticky(client): continue
@@ -73,7 +74,7 @@ class InfoModel:
                 print "DEBUG warning _NET_WM_DESKTOP improperly set"
                 workspace_num = curr_workspace_num
             if self.gconf_client.get_string(cons.GCONF_ONLY_CURR_DESK % glob.screen_index) == cons.STR_TRUE\
-            and (workspace_num != curr_workspace_num or not support.is_window_in_curr_viewport(desktop_width, desktop_height, client)):
+            and (workspace_num != curr_workspace_num or not support.is_window_in_curr_viewport(glob.desktop_width, glob.desktop_height, client)):
                 continue
             if self.gconf_client.get_string(cons.GCONF_NOT_MINIMIZED % glob.screen_index) == cons.STR_TRUE\
             and support.is_window_hidden(client):
@@ -103,7 +104,13 @@ class InfoModel:
             # filter based on process name - NB beppie I'd like something better for this but dont know what!
             if len(process_name) > 6 and process_name[-6:] == "x-tile": continue
             if process_name in cons.PROCESSES_BLACKLIST: continue
-            #print process_name
+            #print "win", pid, process_name
+            #if pid > 0:
+                #ppid = os.popen("ps -p %d -oppid=" % pid).read().strip()
+                #pp_name = support.get_process_name(ppid)
+                #print "pwin", ppid, pp_name
+                #if pp_name in cons.PROCESSES_PARENT_BLACKLIST: continue
+            if support.is_candidate_compiz_desktop(client): continue
             self.process_picklist.add(process_name)
             if process_name not in self.process_blacklist: # user filter
                 win_curr_monitor = screen.get_monitor_at_window(gtk.gdk.window_foreign_new(client))
@@ -174,11 +181,7 @@ class InfoModel:
                                         str(win_geom[3])  ])
             tree_iter = self.liststore.iter_next(tree_iter)
         if undo_ready and undo_snap_vec:
-            undo_snap_str = ""
-            for element in undo_snap_vec:
-                undo_snap_str += ",".join(element) + " "
-            undo_snap_str = undo_snap_str[:-1]
-            self.gconf_client.set_string(cons.GCONF_UNDO % glob.screen_index, undo_snap_str)
+            support.undo_snap_write(self.gconf_client, undo_snap_vec)
         return checked_windows_list
 
     def close_checked_windows(self):
@@ -254,13 +257,23 @@ class XTile:
         retrieves and stores the information about the running desktop self.geometry"""
         # create a variable pointing to the instance of the InfoModel class
         self.store = store
+        # icons generation
+        factory = gtk.IconFactory()
+        for filename in cons.ICONS_FILENAMES:
+            stock_name = filename[:-4]
+            filepath = os.path.join(cons.GLADE_PATH, filename)
+            pixbuf = gtk.gdk.pixbuf_new_from_file(filepath)
+            iconset = gtk.IconSet(pixbuf)
+            factory.add(stock_name, iconset)
+        factory.add_default()
         # system settings
         try:
             gtk_settings = gtk.settings_get_default()
             gtk_settings.set_property("gtk-button-images", True)
             gtk_settings.set_property("gtk-menu-images", True)
         except: pass # older gtk do not have the property "gtk-menu-images"
-        os.environ['UBUNTU_MENUPROXY'] = '0' # for custom stock icons not visible in appmenu
+        os.environ['UBUNTU_MENUPROXY'] = '0' # x-tile has custom stock icons not visible in appmenu
+        self.cmd_line_only = False
         # instantiate the Glade Widgets Wrapper
         self.glade = GladeWidgetsWrapper(cons.GLADE_PATH + 'x-tile.glade', self)
         # ui manager
@@ -308,9 +321,9 @@ class XTile:
         self.glade.scrolledwindow.add(self.view)
         self.glade.processadddialog.connect('key_press_event', self.on_key_press_processadddialog)
         self.glade.aboutdialog.set_version(cons.VERSION)
+        self.glade.window.set_title(self.glade.window.get_title() + " " + cons.VERSION)
         self.gconf_client = gconf.client_get_default()
         self.gconf_client.add_dir("/apps/x-tile/%s" % glob.screen_index, gconf.CLIENT_PRELOAD_NONE)
-        self.statusbar_context_id = self.glade.statusbar.get_context_id('')
         self.combobox_country_lang_init()
 
     def combobox_country_lang_init(self):
@@ -346,15 +359,16 @@ class XTile:
     
     def status_icon_enable(self):
         """Creates the Stats Icon"""
-        if HAS_APPINDICATOR:
+        if self.use_appindicator():
             self.ind = appindicator.Indicator("x-tile", "indicator-messages", appindicator.CATEGORY_APPLICATION_STATUS)
+            self.ind.set_icon_theme_path(cons.GLADE_PATH)
             self.ind.set_status(appindicator.STATUS_ACTIVE)
             self.ind.set_attention_icon("indicator-messages-new")
-            self.ind.set_icon("x-tile")
+            self.ind.set_icon("tile-quad")
             self.ind.set_menu(self.ui.get_widget("/SysTrayMenu"))
         else:
             self.status_icon = gtk.StatusIcon()
-            self.status_icon.set_from_stock("Tile Quad")
+            self.status_icon.set_from_stock("tile-quad")
             self.status_icon.connect('button-press-event', self.on_mouse_button_clicked_systray)
             self.status_icon.set_tooltip(_("Tile the Windows Upon your X Desktop"))
     
@@ -371,11 +385,24 @@ class XTile:
             self.reload_windows_list()
             self.win_on_screen = True
     
+    def on_checkbutton_use_appind_toggled(self, checkbutton):
+        """Use AppIndicator Toggled Handling"""
+        if self.glade.checkbutton_systray_docking.get_active():
+            former_active = True
+            self.glade.checkbutton_systray_docking.set_active(False)
+        else: former_active = False
+        if checkbutton.get_active():
+            self.gconf_client.set_string(cons.GCONF_USE_APPIND % glob.screen_index, cons.STR_TRUE)
+        else:
+            self.gconf_client.set_string(cons.GCONF_USE_APPIND % glob.screen_index, cons.STR_FALSE)
+        if former_active:
+            self.glade.checkbutton_systray_docking.set_active(True)
+    
     def on_checkbutton_systray_docking_toggled(self, checkbutton):
         """SysTray Toggled Handling"""
         self.systray_on = checkbutton.get_active()
         if self.systray_on:
-            if not HAS_APPINDICATOR:
+            if not self.use_appindicator():
                 if "status_icon" in dir(self): self.status_icon.set_property('visible', True)
                 else: self.status_icon_enable()
             else:
@@ -386,7 +413,7 @@ class XTile:
             if self.gconf_client.get_string(cons.GCONF_SYSTRAY_ENABLE % glob.screen_index) != cons.STR_TRUE:
                 self.gconf_client.set_string(cons.GCONF_SYSTRAY_ENABLE % glob.screen_index, cons.STR_TRUE)
         else:
-            if not HAS_APPINDICATOR:
+            if not self.use_appindicator():
                 if not "status_icon" in dir(self): self.status_icon_enable()
                 self.status_icon.set_property('visible', False)
             else:
@@ -462,10 +489,12 @@ class XTile:
         if self.gconf_client.get_string(cons.GCONF_EXIT_AFTER_TILE % glob.screen_index) == None:
             self.gconf_client.set_string(cons.GCONF_EXIT_AFTER_TILE % glob.screen_index, cons.STR_TRUE)
         if self.gconf_client.get_string(cons.GCONF_NOT_MINIMIZED % glob.screen_index) == None:
-            self.gconf_client.set_string(cons.GCONF_NOT_MINIMIZED % glob.screen_index, cons.STR_TRUE)
+            self.gconf_client.set_string(cons.GCONF_NOT_MINIMIZED % glob.screen_index, cons.STR_FALSE)
         if self.gconf_client.get_string(cons.GCONF_ONLY_CURR_DESK % glob.screen_index) == None:
             self.gconf_client.set_string(cons.GCONF_ONLY_CURR_DESK % glob.screen_index, cons.STR_FALSE)
         # systray handling
+        if not HAS_SYSTRAY: self.gconf_client.set_string(cons.GCONF_USE_APPIND % glob.screen_index, cons.STR_TRUE)
+        elif not HAS_APPINDICATOR: self.gconf_client.set_string(cons.GCONF_USE_APPIND % glob.screen_index, cons.STR_FALSE)
         if self.gconf_client.get_string(cons.GCONF_SYSTRAY_ENABLE % glob.screen_index) == None:
             self.gconf_client.set_string(cons.GCONF_SYSTRAY_ENABLE % glob.screen_index, cons.STR_TRUE)
         if self.gconf_client.get_string(cons.GCONF_SYSTRAY_START % glob.screen_index) == None:
@@ -477,6 +506,8 @@ class XTile:
         else:
             self.systray_on = False
             self.ui.get_widget("/MenuBar/FileMenu/ExitApp").set_property('visible', False)
+        # exit after tile
+        self.glade.exit_after_tile_checkbutton.set_active(self.gconf_client.get_string(cons.GCONF_EXIT_AFTER_TILE % glob.screen_index) == cons.STR_TRUE)
         # monitor 1 handling
         if self.gconf_client.get_int(cons.GCONF_OVERRIDE_1 % glob.screen_index) == 1:
             cons.OVERRIDE_1 = 1
@@ -569,14 +600,6 @@ class XTile:
         self.gconf_client.set_int(cons.GCONF_TOOLBAR_ICON_SIZE % glob.screen_index, toolbar_icon_size)
         self.ui.get_widget("/ToolBar").set_property("icon-size", cons.ICONS_SIZE[toolbar_icon_size])
 
-    def update_statusbar(self):
-        """Update the Statusbar after checking checkbuttons"""
-        ON_OFF = {cons.STR_TRUE:_("ON"), cons.STR_FALSE:_("OFF")}
-        statusbar_str = _("Version %s  -  Exit After Tile: %s") %\
-                         (cons.VERSION,
-                          ON_OFF[self.gconf_client.get_string(cons.GCONF_EXIT_AFTER_TILE % glob.screen_index)])
-        self.glade.statusbar.push(self.statusbar_context_id, statusbar_str)
-
     def toggle_active(self, cell, path, model):
         """Toggles the Active state"""
         model[path][0] = not model[path][0]
@@ -593,7 +616,6 @@ class XTile:
             if self.gconf_client.get_string(cons.GCONF_EXIT_AFTER_TILE % glob.screen_index) == cons.STR_TRUE:
                 self.gconf_client.set_string(cons.GCONF_EXIT_AFTER_TILE % glob.screen_index, cons.STR_FALSE)
             else: self.gconf_client.set_string(cons.GCONF_EXIT_AFTER_TILE % glob.screen_index, cons.STR_TRUE)
-            self.update_statusbar()
 
     def toggle_do_not_list_minimized(self, *args):
         """Toggles the flag Do Not List Minimized Windows in List"""
@@ -611,18 +633,6 @@ class XTile:
             else: self.gconf_client.set_string(cons.GCONF_ONLY_CURR_DESK % glob.screen_index, cons.STR_TRUE)
             self.store.load_model(self)
 
-    def toggle_exit_after_tile_keyboard(self):
-        """Toggles the flag Exit After Tile Through Shortcut"""
-        if self.glade.configwindow.get_property("visible") == True:
-            support.dialog_warning(_("The used shortcut is not available when the Preferences Window is opened"), self.glade.window)
-        else: self.toggle_exit_after_tile()
-
-    def toggle_only_curr_workspace_keyboard(self):
-        """Toggles the flag Only Current Workspace Windows in List Through Shortcut"""
-        if self.glade.configwindow.get_property("visible") == True:
-            support.dialog_warning(_("The used shortcut is not available when the Preferences Window is opened"), self.glade.window)
-        else: self.toggle_only_curr_workspace()
-
     def is_window_visible(self):
         """Returns True if the window is visible, False otherwise"""
         return self.glade.window.get_property("visible")
@@ -654,11 +664,13 @@ class XTile:
         """Open the Config Window"""
         if self.glade.configwindow.get_property("visible") == True: return
         self.no_toggling_signals = True
-        self.glade.exit_after_tile_checkbutton.set_active(self.gconf_client.get_string(cons.GCONF_EXIT_AFTER_TILE % glob.screen_index) == cons.STR_TRUE)
         self.glade.current_workspace_checkbutton.set_active(self.gconf_client.get_string(cons.GCONF_ONLY_CURR_DESK % glob.screen_index) == cons.STR_TRUE)
         self.glade.do_not_list_minimized_checkbutton.set_active(self.gconf_client.get_string(cons.GCONF_NOT_MINIMIZED % glob.screen_index) == cons.STR_TRUE)
         self.glade.checkbutton_systray_docking.set_active(self.gconf_client.get_string(cons.GCONF_SYSTRAY_ENABLE % glob.screen_index) == cons.STR_TRUE)
         self.glade.checkbutton_start_minimized.set_active(self.gconf_client.get_string(cons.GCONF_SYSTRAY_START % glob.screen_index) == cons.STR_TRUE)
+        self.glade.checkbutton_use_appind.set_active(self.gconf_client.get_string(cons.GCONF_USE_APPIND % glob.screen_index) == cons.STR_TRUE)
+        if not HAS_APPINDICATOR or not HAS_SYSTRAY:
+            self.glade.checkbutton_use_appind.set_sensitive(False)
         self.glade.show_toolbar_checkbutton.set_active(self.gconf_client.get_string(cons.GCONF_SHOW_TOOLBAR % glob.screen_index) == cons.STR_TRUE)
         self.no_toggling_signals = False
         self.glade.configwindow.show_all()
@@ -883,9 +895,14 @@ class XTile:
         self.gconf_client.set_int(cons.GCONF_GRID_COLS % glob.screen_index, cons.GRID_COLS)
         self.tile_grid()
 
+    def use_appindicator(self):
+        """Will we use AppIndicator?"""
+        return self.gconf_client.get_string(cons.GCONF_USE_APPIND % glob.screen_index) == cons.STR_TRUE
+
     def quit_application_totally(self, *args):
         """The process is Shut Down"""
-        if not HAS_APPINDICATOR and "status_icon" in dir(self): self.status_icon.set_visible(False)
+        if not self.use_appindicator() and "status_icon" in dir(self):
+            self.status_icon.set_visible(False)
         self.save_win_pos_n_size()
         self.glade.window.destroy()
         gtk.main_quit()
@@ -917,9 +934,12 @@ class XTile:
     def launch_application(self):
         """Show the main window and all child widgets"""
         self.init_from_gconf()
-        self.update_statusbar()
         self.window_position_restore()
         self.glade.window.show_all()
+        if glob.is_compiz_running:
+            self.glade.checkbutton_dest_workspace.set_active(False)
+            self.glade.checkbutton_dest_workspace.hide()
+            self.glade.spinbutton_dest_workspace.hide()
         show_toolbar = self.gconf_client.get_string(cons.GCONF_SHOW_TOOLBAR % glob.screen_index)
         if show_toolbar == None: self.gconf_client.set_string(cons.GCONF_SHOW_TOOLBAR % glob.screen_index, cons.STR_TRUE)
         elif show_toolbar == cons.STR_FALSE: self.ui.get_widget("/ToolBar").hide()
@@ -964,28 +984,32 @@ class XTile:
         """Tile the Checked Windows Vertically"""
         self.gconf_client.set_string(cons.GCONF_LATEST_TILING % glob.screen_index, "v")
         checked_windows_list = self.store.get_checked_windows_list(True)
-        tilings.tile_vertically(checked_windows_list, glob.monitors_areas)
+        tilings.tile_vertically(checked_windows_list, glob.monitors_areas, self.get_dest_ws())
         self.check_exit_after_tile()
 
     def tile_horizontally(self, *args):
         """Tile the Checked Windows Horizontally"""
         self.gconf_client.set_string(cons.GCONF_LATEST_TILING % glob.screen_index, "h")
         checked_windows_list = self.store.get_checked_windows_list(True)
-        tilings.tile_horizontally(checked_windows_list, glob.monitors_areas)
+        tilings.tile_horizontally(checked_windows_list, glob.monitors_areas, self.get_dest_ws())
         self.check_exit_after_tile()
 
     def tile_quad(self, *args):
         """Tile the Checked Windows Quad"""
         self.gconf_client.set_string(cons.GCONF_LATEST_TILING % glob.screen_index, "q")
         checked_windows_list = self.store.get_checked_windows_list(True)
-        tilings.tile_quad(checked_windows_list, glob.monitors_areas)
+        tilings.tile_quad(checked_windows_list, glob.monitors_areas, self.get_dest_ws())
         self.check_exit_after_tile()
 
     def tile_grid(self):
         """Tile the Checked Windows in a rows by cols grid"""
         self.gconf_client.set_string(cons.GCONF_LATEST_TILING % glob.screen_index, "g")
         checked_windows_list = self.store.get_checked_windows_list(True)
-        tilings.tile_grid(cons.GRID_ROWS, cons.GRID_COLS, checked_windows_list, glob.monitors_areas)
+        tilings.tile_grid(cons.GRID_ROWS,
+                          cons.GRID_COLS,
+                          checked_windows_list,
+                          glob.monitors_areas,
+                          self.get_dest_ws())
         self.check_exit_after_tile()
 
     def tile_custom_1_set(self, *args):
@@ -1059,6 +1083,12 @@ class XTile:
             if rgb_idx + 1 < len(cons.DRAW_RGBS): rgb_idx += 1
             else: rgb_idx = 0
 
+    def get_dest_ws(self):
+        """Get Destination Desktop"""
+        if self.glade.checkbutton_dest_workspace.get_active():
+            return int(self.glade.spinbutton_dest_workspace.get_value()-1)
+        return -1
+
     def invert_tiling(self, *args):
         """Invert the order of the latest tiling operation"""
         # get the win_id and win_geom of the latest tiled windows
@@ -1066,6 +1096,7 @@ class XTile:
         undo_snap_str = self.gconf_client.get_string(cons.GCONF_UNDO % glob.screen_index)
         if not undo_snap_str: return
         undo_snap_vec = undo_snap_str.split(" ")
+        doubleundo_snap_vec = []
         for element in undo_snap_vec:
             win_id, is_maximized, x, y, width, height = element.split(",")
             win_geom = support.get_geom(int(win_id))
@@ -1079,11 +1110,47 @@ class XTile:
         #print "next_tiling_geoms", next_tiling_geoms
         # tile the windows
         for element in next_tiling_geoms:
+            doubleundo_snap_vec.append(support.get_undo_element_from_win_id(int(element['win_id'])))
             support.moveresize(int(element['win_id']),
                                int(element['win_geom'][0]),
                                int(element['win_geom'][1]),
                                int(element['win_geom'][2]),
-                               int(element['win_geom'][3]))
+                               int(element['win_geom'][3]),
+                               self.get_dest_ws())
+        if doubleundo_snap_vec:
+            support.undo_snap_write(self.gconf_client, doubleundo_snap_vec)
+        self.check_exit_after_tile()
+
+    def cycle_tiling(self, *args):
+        """Cycle the order of the latest tiling operation"""
+        # get the win_id and win_geom of the latest tiled windows
+        latest_tiling_geoms = []
+        undo_snap_str = self.gconf_client.get_string(cons.GCONF_UNDO % glob.screen_index)
+        if not undo_snap_str: return
+        undo_snap_vec = undo_snap_str.split(" ")
+        doubleundo_snap_vec = []
+        for element in undo_snap_vec:
+            win_id, is_maximized, x, y, width, height = element.split(",")
+            win_geom = support.get_geom(int(win_id))
+            latest_tiling_geoms.append({'win_id': win_id, 'win_geom': win_geom})
+        #print "latest_tiling_geoms", latest_tiling_geoms
+        # generate the win_id and win_geom of the next tiled windows
+        next_tiling_geoms = []
+        for i, element in enumerate(latest_tiling_geoms):
+            next_tiling_geoms.append({'win_id': element['win_id'],
+                                      'win_geom': latest_tiling_geoms[(i+1)%len(latest_tiling_geoms)]['win_geom']})
+        #print "next_tiling_geoms", next_tiling_geoms
+        # tile the windows
+        for element in next_tiling_geoms:
+            doubleundo_snap_vec.append(support.get_undo_element_from_win_id(int(element['win_id'])))
+            support.moveresize(int(element['win_id']),
+                               int(element['win_geom'][0]),
+                               int(element['win_geom'][1]),
+                               int(element['win_geom'][2]),
+                               int(element['win_geom'][3]),
+                               self.get_dest_ws())
+        if doubleundo_snap_vec:
+            support.undo_snap_write(self.gconf_client, doubleundo_snap_vec)
         self.check_exit_after_tile()
 
     def undo_tiling(self, *args):
@@ -1095,25 +1162,13 @@ class XTile:
         for element in undo_snap_vec:
             win_id, is_maximized, x, y, width, height = element.split(",")
             # save current state for eventual undo of the undo
-            win_id = int(win_id)
-            if support.is_window_Vmax(win_id) or support.is_window_Hmax(win_id): is_maximized = 1
-            else: is_maximized = 0
-            win_geom = support.get_geom(win_id)
-            doubleundo_snap_vec.append([  str(win_id),
-                                          str(is_maximized),
-                                          str(win_geom[0]),
-                                          str(win_geom[1]),
-                                          str(win_geom[2]),
-                                          str(win_geom[3])  ])
+            win_id_int = int(win_id)
+            doubleundo_snap_vec.append(support.get_undo_element_from_win_id(win_id_int))
             # proceed with the undo
-            if int(is_maximized) == 1: support.maximize(int(win_id))
-            else: support.moveresize(win_id, int(x), int(y), int(width), int(height))
+            if int(is_maximized) == 1: support.maximize(win_id_int)
+            else: support.moveresize(win_id_int, int(x), int(y), int(width), int(height), self.get_dest_ws())
         if doubleundo_snap_vec:
-            doubleundo_snap_str = ""
-            for element in doubleundo_snap_vec:
-                doubleundo_snap_str += ",".join(element) + " "
-            doubleundo_snap_str = doubleundo_snap_str[:-1]
-            self.gconf_client.set_string(cons.GCONF_UNDO % glob.screen_index, doubleundo_snap_str)
+            support.undo_snap_write(self.gconf_client, doubleundo_snap_vec)
         self.check_exit_after_tile()
 
     def tile_custom_1_run(self, *args):
@@ -1129,7 +1184,7 @@ class XTile:
         win_num = 0
         for checked_window in windows_list:
             x, y, width, height = custom_geoms_vec[win_num].split(",")
-            support.moveresize(checked_window, int(x), int(y), int(width), int(height))
+            support.moveresize(checked_window, int(x), int(y), int(width), int(height), self.get_dest_ws())
             if win_num + 1 < len(custom_geoms_vec): win_num += 1
             else: break
         self.check_exit_after_tile()
@@ -1147,7 +1202,7 @@ class XTile:
         win_num = 0
         for checked_window in windows_list:
             x, y, width, height = custom_geoms_vec[win_num].split(",")
-            support.moveresize(checked_window, int(x), int(y), int(width), int(height))
+            support.moveresize(checked_window, int(x), int(y), int(width), int(height), self.get_dest_ws())
             if win_num + 1 < len(custom_geoms_vec): win_num += 1
             else: break
         self.check_exit_after_tile()
@@ -1156,28 +1211,28 @@ class XTile:
         """Tile 3 Windows in Triangle Up Scheme"""
         self.gconf_client.set_string(cons.GCONF_LATEST_TILING % glob.screen_index, "u")
         checked_windows_list = self.store.get_checked_windows_list(True)
-        tilings.tile_triangle_up(checked_windows_list, glob.monitors_areas)
+        tilings.tile_triangle_up(checked_windows_list, glob.monitors_areas, self.get_dest_ws())
         self.check_exit_after_tile()
 
     def tile_triangle_down(self, *args):
         """Tile 3 Windows in Triangle Down Scheme"""
         self.gconf_client.set_string(cons.GCONF_LATEST_TILING % glob.screen_index, "d")
         checked_windows_list = self.store.get_checked_windows_list(True)
-        tilings.tile_triangle_down(checked_windows_list, glob.monitors_areas)
+        tilings.tile_triangle_down(checked_windows_list, glob.monitors_areas, self.get_dest_ws())
         self.check_exit_after_tile()
 
     def tile_triangle_left(self, *args):
         """Tile 3 Windows in Triangle Left Scheme"""
         self.gconf_client.set_string(cons.GCONF_LATEST_TILING % glob.screen_index, "l")
         checked_windows_list = self.store.get_checked_windows_list(True)
-        tilings.tile_triangle_left(checked_windows_list, glob.monitors_areas)
+        tilings.tile_triangle_left(checked_windows_list, glob.monitors_areas, self.get_dest_ws())
         self.check_exit_after_tile()
 
     def tile_triangle_right(self, *args):
         """Tile 3 Windows in Triangle Right Scheme"""
         self.gconf_client.set_string(cons.GCONF_LATEST_TILING % glob.screen_index, "r")
         checked_windows_list = self.store.get_checked_windows_list(True)
-        tilings.tile_triangle_right(checked_windows_list, glob.monitors_areas)
+        tilings.tile_triangle_right(checked_windows_list, glob.monitors_areas, self.get_dest_ws())
         self.check_exit_after_tile()
 
     def show_hide_toolbar(self, menuitem, data=None):
@@ -1192,6 +1247,7 @@ class XTile:
     
     def check_exit_after_tile(self):
         """Check if the Exit After Tile is Active and Eventually Quit"""
+        if self.cmd_line_only: return
         if self.gconf_client.get_string(cons.GCONF_EXIT_AFTER_TILE % glob.screen_index) == cons.STR_TRUE:
             self.quit_application()
     
@@ -1217,6 +1273,10 @@ class XTile:
         """Invert the Order of the Latest Tiling Operation"""
         self.hide_and_process("x-tile i &")
     
+    def cycle_tiling_all(self, *args):
+        """Cycle the Order of the Latest Tiling Operation"""
+        self.hide_and_process("x-tile y &")
+    
     def tile_all_vertically(self, *args):
         """Just tile Vertically all opened windows"""
         self.hide_and_process("x-tile v &")
